@@ -6,18 +6,6 @@ from functools import partial
 from picomc.logging import logger
 
 
-class cached_property(object):
-    def __init__(self, fn):
-        self.fn = fn
-
-    def __get__(self, inst, cls):
-        if inst is None:
-            return self
-        r = self.fn(inst)
-        setattr(inst, self.fn.__name__, r)
-        return r
-
-
 def join_classpath(*cp):
     return os.pathsep.join(cp)
 
@@ -37,3 +25,9 @@ def die(mesg, code=1):
 
 def sanitize_name(name):
     return name.replace("..", "_").replace("/", "_")
+
+
+def recur_files(path):
+    for dirpath, dirnames, filenames in os.walk(path):
+        for f in filenames:
+            yield os.path.join(dirpath, f)
