@@ -1,4 +1,6 @@
+import enum
 import json
+import operator
 import os
 import posixpath
 import shutil
@@ -8,83 +10,40 @@ from functools import reduce
 
 import requests
 from picomc.downloader import DownloadQueue
-from picomc.env import Env, file_verify_relative, get_filepath
+from picomc.env import Env, get_filepath
 from picomc.javainfo import get_java_info
 from picomc.library import Library
 from picomc.logging import logger
 from picomc.rules import match_ruleset
-from picomc.utils import cached_property, die, file_sha1
+from picomc.utils import die, file_sha1, recur_files
 
 
-class VersionType:
-    MAP = {"release": 1, "snapshot": 2, "old_alpha": 4, "old_beta": 8}
-
-    def __init__(self, d=None):
-        if isinstance(d, int):
-            self._a = d
-        elif isinstance(d, str):
-            self._a = self.MAP[d]
-
-    def __or__(self, other):
-        return VersionType(self._a | other._a)
+class VersionType(enum.Flag):
+    NONE = 0
+    RELEASE = enum.auto()
+    SNAPSHOT = enum.auto()
+    ALPHA = enum.auto()
+    BETA = enum.auto()
+    ANY = RELEASE | SNAPSHOT | ALPHA | BETA
 
     def match(self, s):
-        return bool(self.MAP[s] & self._a)
+        names = {
+            "release": VersionType.RELEASE,
+            "snapshot": VersionType.SNAPSHOT,
+            "old_alpha": VersionType.ALPHA,
+            "old_beta": VersionType.BETA,
+        }
+        return bool(names[s] & self)
 
     @staticmethod
     def create(release, snapshot, alpha, beta):
-        a = release | (2 * snapshot) | (4 * alpha) | (8 * beta)
-        return VersionType(a)
-
-
-VersionType.RELEASE = VersionType("release")
-VersionType.SNAPSHOT = VersionType("snapshot")
-VersionType.ALPHA = VersionType("old_alpha")
-VersionType.BETA = VersionType("old_beta")
-
-MODE_OVERRIDE = 0
-MODE_REDUCE = 1
-
-
-class CachedVspecAttr(object):
-    def __init__(self, attr, mode=MODE_OVERRIDE, reduce_func=None, default=None):
-        self.attr = attr
-        self.mode = mode
-        self.rfunc = reduce_func
-        self.default = default
-
-    def __get__(self, vspec, cls):
-        if vspec is None:
-            return self
-        try:
-            if self.mode == MODE_OVERRIDE:
-                for v in vspec.chain:
-                    if self.attr in v.raw_vspec:
-                        r = v.raw_vspec[self.attr]
-                        break
-                else:
-                    raise AttributeError()
-            elif self.mode == MODE_REDUCE:
-                try:
-                    r = reduce(
-                        self.rfunc,
-                        (
-                            v.raw_vspec[self.attr]
-                            for v in vspec.chain[::-1]
-                            if self.attr in v.raw_vspec
-                        ),
-                    )
-                except TypeError as e:
-                    raise AttributeError() from e
-        except AttributeError:
-            if self.default is not None:
-                r = self.default(vspec.vobj)
-        finally:
-            try:
-                setattr(vspec, self.attr, r)
-                return r
-            except UnboundLocalError as e:
-                raise AttributeError() from e
+        D = {
+            VersionType.RELEASE: release,
+            VersionType.SNAPSHOT: snapshot,
+            VersionType.ALPHA: alpha,
+            VersionType.BETA: beta,
+        }.items()
+        return reduce(operator.or_, (k for k, v in D if v), VersionType.NONE)
 
 
 def argumentadd(d1, d2):
@@ -98,92 +57,111 @@ def argumentadd(d1, d2):
 
 
 class VersionSpec:
-    def __init__(self, vobj):
+    def __init__(self, vobj, version_manager):
         self.vobj = vobj
-        self.chain = self._resolve_chain()
+        self.chain = self.resolve_chain(version_manager)
+        self.initialize_fields()
 
-    def _resolve_chain(self):
+    def resolve_chain(self, version_manager):
         chain = []
         chain.append(self.vobj)
         cv = self.vobj
         while "inheritsFrom" in cv.raw_vspec:
-            cv = Version(cv.raw_vspec["inheritsFrom"])
+            cv = version_manager.get_version(cv.raw_vspec["inheritsFrom"])
             chain.append(cv)
         return chain
 
-    minecraftArguments = CachedVspecAttr("minecraftArguments")
-    arguments = CachedVspecAttr("arguments", mode=MODE_REDUCE, reduce_func=argumentadd)
-    mainClass = CachedVspecAttr("mainClass")
-    assetIndex = CachedVspecAttr("assetIndex")
-    libraries = CachedVspecAttr(
-        "libraries", mode=MODE_REDUCE, reduce_func=lambda x, y: y + x
-    )
-    jar = CachedVspecAttr("jar", default=lambda vobj: vobj.version_name)
-    downloads = CachedVspecAttr("downloads", default=lambda vobj: {})
+    def attr_override(self, attr, default=None):
+        for v in self.chain:
+            if attr in v.raw_vspec:
+                return v.raw_vspec[attr]
+        if default is None:
+            raise AttributeError(attr)
+        return default
+
+    def attr_reduce(self, attr, reduce_func):
+        L = [v.raw_vspec[attr] for v in self.chain[::-1] if attr in v.raw_vspec]
+        if not L:
+            raise AttributeError(attr)
+        return reduce(reduce_func, L)
+
+    def initialize_fields(self):
+        try:
+            self.minecraftArguments = self.attr_override("minecraftArguments")
+        except AttributeError:
+            pass
+        try:
+            self.arguments = self.attr_reduce("arguments", argumentadd)
+        except AttributeError:
+            pass
+        self.mainClass = self.attr_override("mainClass")
+        self.assetIndex = self.attr_override("assetIndex")
+        self.assets = self.attr_override("assets")
+        self.libraries = self.attr_reduce("libraries", lambda x, y: y + x)
+        self.jar = self.attr_override("jar", default=self.vobj.version_name)
+        self.downloads = self.attr_override("downloads", default={})
 
 
 class Version:
     ASSETS_URL = "http://resources.download.minecraft.net/"
 
-    def __init__(self, version_name):
+    def __init__(self, version_name, version_manager, version_manifest):
         self.version_name = version_name
+        self.version_manager = version_manager
+        self.version_manifest = version_manifest
         self._libraries = dict()
 
-    @cached_property
-    def jarfile(self):
-        v = self.vspec.jar
-        return get_filepath("versions", v, "{}.jar".format(v))
+        self.raw_vspec = self.get_raw_vspec()
+        self.vspec = VersionSpec(self, self.version_manager)
 
-    @cached_property
-    def raw_vspec(self):
-        fpath = get_filepath(
+        self.raw_asset_index = self.get_raw_asset_index(self.vspec)
+
+        jarname = self.vspec.jar
+        self.jarfile = get_filepath("versions", jarname, "{}.jar".format(jarname))
+
+    def get_raw_vspec(self):
+        vspec_path = get_filepath(
             "versions", self.version_name, "{}.json".format(self.version_name)
         )
-        ver = Env.vm.get_manifest_version(self.version_name)
-        if not ver:
-            if os.path.exists(fpath):
+        if not self.version_manifest:
+            if os.path.exists(vspec_path):
                 logger.debug("Found custom vspec ({})".format(self.version_name))
-                with open(fpath) as fp:
+                with open(vspec_path) as fp:
                     return json.load(fp)
             else:
-                die("Specified version not available")
-        url = ver["url"]
+                die("Specified version ({}) not available".format(self.version_name))
+        url = self.version_manifest["url"]
         # Pull the hash out of the url. This is prone to breakage, maybe
         # just try to download the vspec and don't care about whether it
         # is up to date or not.
         url_split = urllib.parse.urlsplit(url)
         sha1 = posixpath.basename(posixpath.dirname(url_split.path))
 
-        if os.path.exists(fpath) and file_sha1(fpath) == sha1:
+        if os.path.exists(vspec_path) and file_sha1(vspec_path) == sha1:
             logger.debug(
                 "Using cached vspec files, hash matches manifest ({})".format(
                     self.version_name
                 )
             )
-            with open(fpath) as fp:
+            with open(vspec_path) as fp:
                 return json.load(fp)
 
         try:
             logger.debug("Downloading vspec file")
             raw = requests.get(url).content
-            dirpath = os.path.dirname(fpath)
+            dirpath = os.path.dirname(vspec_path)
             os.makedirs(dirpath, exist_ok=True)
-            with open(fpath, "wb") as fp:
+            with open(vspec_path, "wb") as fp:
                 fp.write(raw)
             j = json.loads(raw)
             return j
         except requests.ConnectionError:
             die("Failed to retrieve version json file. Check your internet connection.")
 
-    @cached_property
-    def vspec(self):
-        return VersionSpec(self)
-
-    @cached_property
-    def raw_asset_index(self):
-        iid = self.vspec.assetIndex["id"]
-        url = self.vspec.assetIndex["url"]
-        sha1 = self.vspec.assetIndex["sha1"]
+    def get_raw_asset_index(self, vspec):
+        iid = vspec.assetIndex["id"]
+        url = vspec.assetIndex["url"]
+        sha1 = vspec.assetIndex["sha1"]
         fpath = get_filepath("assets", "indexes", "{}.json".format(iid))
         if os.path.exists(fpath) and file_sha1(fpath) == sha1:
             logger.debug("Using cached asset index, hash matches vspec")
@@ -199,7 +177,10 @@ class Version:
             die("Failed to retrieve asset index.")
 
     def get_libraries(self, java_info):
-        key = java_info.get("java.home", None)
+        if java_info is not None:
+            key = java_info.get("java.home", None)
+        else:
+            key = None
         if key and key in self._libraries:
             return self._libraries[key]
         else:
@@ -215,7 +196,7 @@ class Version:
                 self._libraries[key] = libs
             return libs
 
-    def get_jarfile_dl(self, force=False):
+    def get_jarfile_dl(self, verify_hashes=False, force=False):
         """Checks existence and hash of cached jar. Returns None if ok, otherwise
         returns download (url, size)"""
         logger.debug("Attempting to use jarfile: {}".format(self.jarfile))
@@ -230,7 +211,11 @@ class Version:
         if (
             force
             or not os.path.exists(self.jarfile)
-            or file_sha1(self.jarfile) != dlspec["sha1"]
+            # The fabric-installer places an empty jarfile here, due to some
+            # quirk of an old (git blame 2 years) version of the vanilla launcher.
+            # https://github.com/FabricMC/fabric-installer/blob/master/src/main/java/net/fabricmc/installer/client/ClientInstaller.java#L49
+            or os.path.getsize(self.jarfile) == 0
+            or (verify_hashes and file_sha1(self.jarfile) != dlspec["sha1"])
         ):
             logger.info(
                 "Jar file ({}) will be downloaded with libraries.".format(
@@ -239,7 +224,7 @@ class Version:
             )
             return dlspec["url"], dlspec.get("size", None)
 
-    def download_libraries(self, java_info, force=False):
+    def download_libraries(self, java_info, verify_hashes=False, force=False):
         """Downloads missing libraries."""
         logger.info("Checking libraries.")
         q = DownloadQueue()
@@ -249,7 +234,7 @@ class Version:
             basedir = get_filepath("libraries")
             abspath = library.get_abspath(basedir)
             ok = os.path.isfile(abspath) and os.path.getsize(abspath) > 0
-            if library.sha1 is not None:
+            if verify_hashes and library.sha1 is not None:
                 ok = ok and file_sha1(abspath) == library.sha1
             if not ok and not library.url:
                 logger.error(
@@ -258,7 +243,7 @@ class Version:
                 continue
             if force or not ok:
                 q.add(library.url, library.get_abspath(basedir), library.size)
-        jardl = self.get_jarfile_dl()
+        jardl = self.get_jarfile_dl(verify_hashes, force)
         if jardl is not None:
             url, size = jardl
             q.add(url, self.jarfile, size=size)
@@ -289,7 +274,7 @@ class Version:
             logger.debug("Resources path: {}".format(where))
             self._populate_virtual_assets(where)
 
-    def download_assets(self, force=False):
+    def download_assets(self, verify_hashes=False, force=False):
         """Downloads missing assets."""
 
         hashes = dict()
@@ -300,18 +285,24 @@ class Version:
 
         is_virtual = self.raw_asset_index.get("virtual", False)
 
+        fileset = set(recur_files(get_filepath("assets")))
         q = DownloadQueue()
+        objpath = get_filepath("assets", "objects")
         for sha in hashes:
-            path = os.path.join("assets", "objects", sha[0:2], sha)
-            if file_verify_relative(path, sha):
-                continue
-            url = urllib.parse.urljoin(self.ASSETS_URL, posixpath.join(sha[0:2], sha))
-            q.add(url, os.path.join(Env.app_root, path), size=hashes[sha])
+            abspath = os.path.join(objpath, sha[0:2], sha)
+            ok = abspath in fileset  # file exists
+            if verify_hashes:
+                ok = ok and file_sha1(abspath) == sha
+            if force or not ok:
+                url = urllib.parse.urljoin(
+                    self.ASSETS_URL, posixpath.join(sha[0:2], sha)
+                )
+                q.add(url, abspath, size=hashes[sha])
 
         if len(q) > 0:
             logger.info("Downloading {} assets.".format(len(q)))
         if not q.download():
-            logger.error("Some assets failed to download.")
+            logger.warn("Some assets failed to download.")
 
         if is_virtual:
             logger.info("Copying virtual assets")
@@ -319,21 +310,22 @@ class Version:
             logger.debug("Virtual asset path: {}".format(where))
             self._populate_virtual_assets(where)
 
-    def prepare(self, java_info=None):
+    def prepare(self, java_info=None, verify_hashes=False):
         if not java_info:
             java_info = get_java_info(Env.gconf.get("java.path"))
-        self.download_libraries(java_info)
-        self.download_assets()
+        self.download_libraries(java_info, verify_hashes)
+        self.download_assets(verify_hashes)
 
-    def prepare_launch(self, gamedir, java_info):
-        self.prepare(java_info)
+    def prepare_launch(self, gamedir, java_info, verify_hahes=False):
+        self.prepare(java_info, verify_hahes)
         self.prepare_assets_launch(gamedir)
 
 
 class VersionManager:
-    VERSION_MANIFEST_URL = (
-        "https://launchermeta.mojang.com/mc/game/version_manifest.json"
-    )
+    MANIFEST_URL = "https://launchermeta.mojang.com/mc/game/version_manifest.json"
+
+    def __init__(self):
+        self.manifest = self.get_manifest()
 
     def resolve_version_name(self, v):
         """Takes a metaversion and resolves to a version."""
@@ -345,11 +337,10 @@ class VersionManager:
             logger.debug("Resolved snapshot -> {}".format(v))
         return v
 
-    @cached_property
-    def manifest(self):
+    def get_manifest(self):
         manifest_filepath = get_filepath("versions", "manifest.json")
         try:
-            m = requests.get(self.VERSION_MANIFEST_URL).json()
+            m = requests.get(self.MANIFEST_URL).json()
             with open(manifest_filepath, "w") as mfile:
                 json.dump(m, mfile, indent=4, sort_keys=True)
             return m
@@ -366,11 +357,6 @@ class VersionManager:
                 logger.warn("Cached version manifest not available.")
                 raise RuntimeError("Failed to retrieve version manifest.")
 
-    def get_manifest_version(self, version_name):
-        for ver in self.manifest["versions"]:
-            if ver["id"] == version_name:
-                return ver
-
     def version_list(self, vtype=VersionType.RELEASE, local=False):
         r = [v["id"] for v in self.manifest["versions"] if vtype.match(v["type"])]
         if local:
@@ -384,4 +370,10 @@ class VersionManager:
         return r
 
     def get_version(self, version_name):
-        return Version(self.resolve_version_name(version_name))
+        name = self.resolve_version_name(version_name)
+        version_manifest = None
+        for ver in self.manifest["versions"]:
+            if ver["id"] == name:
+                version_manifest = ver
+                break
+        return Version(name, self, version_manifest)
